@@ -827,6 +827,9 @@ PortableServer_POA_deactivate_object(PortableServer_POA obj,
 				CORBA_FALSE,
 				CORBA_FALSE,
 				ev);
+	} else {
+	  PortableServer_ServantBase *serv = oldobj->servant;
+	  serv->_private = NULL;
 	}
 
 	CORBA_free(oldobj->object_id);
